import java.lang.*;

/*
-------------------------------------------------------------------------------
Metabolisme de la vaca
-------------------------------------------------------------------------------
Dades: dt (mida del pas en dies), imax (nombre de passos a calcular),
       jmax (nombre d'iteracions per estabilitzar), qp (activitat de
       l'herba).
Resultats: Al fitxer "vaca.dat". Cada fila representa un instant de temps.
           La primera columna es el temps transcorregut en dies. La seguent
           es l'activitat en Bq a l'interior del tub digestiu. La tercera
           es la contaminacio a la carn en Bq/Kg. La quarta es l'activitat
           al fetge en Bq i la cinquena la concentracio a la llet en Bq/l.
Dades de prova: dt=0.05 imax=140 jmax=2 qp=5000 ==>
                0.05 423.635 0.00130375 0.0170625 0.0189 3.30019
                0.1 821.271 0.00505146 0.0661097 0.073242 3.30019
                0.15 1194.5 0.0110923 0.145167 0.160851 3.30019
                0.2 1544.83 0.0192845 0.252381 0.279685 3.30019
                ..  ......   .......   ......   ......   .......
Observacions: Des del punt de vista teoric tan sols jmax=1 esta justificat.
              Llavors el metode d'Euler es converteix en un Runge-Kutta.
              Les unitats venen fixades per les variables del programa que
              indiquen la massa de la vaca i el seu volum de llet.
-------------------------------------------------------------------------------
*/

public class VacaCalculs {
	double dt,qp,dur,temps[],conc[][];
	int imax;

	VacaCalculs(float dt0,int imax0,float qp0,float dur0) {
		double dx[],x0[],x[],xf[],xm[],font[],mat[][],t0,t,tf,tm,cont,k66,k67,k68,k69,k77,k88,lambda,vo,cm,qf,fd,in;
		int i,j,k,l,jmax;

		dt = (double) dt0;
		imax = imax0;
		qp = (double) qp0;
		dur = (double) dur0;

		dx = new double[4];
		x0 = new double[4];
		x = new double[4];
		xf = new double[4];
		xm = new double[4];
		font = new double[4];
		mat = new double[4][4];

		temps = new double[imax];
		conc = new double[imax][4];

		jmax = 1;

/* Calculem la entrada. Una vaca menja uns 5 Kg diaris en pes sec */

		qf=5.0d;

/* Ara introduim la digestibilitat del cesi: 0.07 per l'herba i 0.2 per
la palla */

		fd=0.07d;

/* I ja es pot calcular l'entrada */

		in=5.0d*qf*fd*qp;

/* Suposem que inicialment no hi ha contaminacio */

		for(k=0;k<4;k++)
			x0[k]=0.0d;
		t0=0.0d;

/* Inicialitzem MAT i FONT ... */

		for(k=0;k<4;k++)for(l=0;l<4;l++)
			mat[k][l]=0.0d;
		for(k=0;k<4;k++)
			font[k]=0.0d;

/* ... i les omplim amb parametres empirics */

		lambda=6.33E-5d;

		k66=1.12E0d;
		k67=5.96E-2d;
		k68=1.56E-3d;
		k69=8.64E-2d;
		k77=6.91E-3d;
		k88=6.91E-3d;

		mat[0][0]=-k66-k67-k68-k69-lambda;
		font[0]=in;
		mat[1][0]=k67;
		mat[1][1]=-k77-lambda;
		mat[2][0]=k68;
		mat[2][2]=-k88-lambda;
		mat[3][0]=k69;
		mat[3][3]=-lambda;

		vo=50.0d;
		cm=500.0d;

/* Ara inicialitzem les matrius */

		for(i=0;i<4;i++) {
			x[i]=x0[i];
			xf[i]=x0[i];
		}
		t=t0;

		for(i=0;i<imax;i++) {
			tm=t+dt/2.0d;
			for(j=0;j<jmax;j++) {
				for(k=0;k<4;k++)
					xm[k]=(x[k]+xf[k])/2.0d;
				for(k=0;k<4;k++) {
					cont=0.0d;
					for(l=0;l<4;l++)
						cont=cont+mat[k][l]*xm[l];
					if(tm<dur)
						cont=cont+font[k];
					dx[k]=cont*dt;
					xf[k]=x[k]+dx[k];
				}
			}
			for(k=0;k<4;k++)
				x[k]=xf[k];
			t=t+dt;
			temps[i] = t;
			for(k=0;k<4;k++)
				conc[i][k] = x[k];
			conc[i][1] = conc[i][1]/cm;
			conc[i][3] = conc[i][3]/vo;
		}
	}
}
